from colorama import Fore, Style

BRIGHTYELLOW = '\u001b[93;1m'

ASCII_ART = {
    'day-drizzle': [
        f'   .-.    ',
        f'  (   ).  ',
        f' (___(__) ',
        f"{Fore.BLUE}.'.'.'.'  {Style.RESET_ALL}",
        f"{Fore.BLUE}.'.'.'.   {Style.RESET_ALL}",
    ],
    'night-drizzle': [
        f'{Fore.LIGHTBLACK_EX}   .-.    {Style.RESET_ALL}',
        f'{Fore.LIGHTBLACK_EX}  (   ).  {Style.RESET_ALL}',
        f'{Fore.LIGHTBLACK_EX} (___(__) {Style.RESET_ALL}',
        f"{Fore.BLUE}.'.'.'.'  {Style.RESET_ALL}",
        f"{Fore.BLUE}.'.'.'.   {Style.RESET_ALL}",
    ],
    'mist': [
        '_ - _ - _ -',
        '_ - _ - _  ',
        '_ - _ - _ -',
    ],
    'night-clear': [
        '     .-"--.   *',
        '+  /   ,’      ',
        '  |    |     @ ',
        '   \\    `.__.  ',
        '  * `-.__.-’   '
    ],
    'night-partial': [
        f'     .-"--.  * ',
        f'*  /   ,’      ',
        f"  |   '{Fore.LIGHTBLACK_EX}.--.   {Style.RESET_ALL}@",
        f'   \\{Fore.LIGHTBLACK_EX}.-(    ).  {Style.RESET_ALL}',
        f'   {Fore.LIGHTBLACK_EX}(___.__)__) {Style.RESET_ALL}'
    ],
    'night-partial-thunderstorm': [
        f'      .-"--.    *',
        f' *  /   ,’      ',
        f'   |    {Fore.LIGHTBLACK_EX}.--.   {Style.RESET_ALL}@',
        f'+   \\{Fore.LIGHTBLACK_EX}.-(    ).  {Style.RESET_ALL}',
        f'   *{Fore.LIGHTBLACK_EX}(___.__)__) {Style.RESET_ALL}',
        f'   {Fore.BLUE}‚‘{Style.RESET_ALL}//{Fore.BLUE}‚‘‚‘{Style.RESET_ALL}/{Fore.BLUE}‚‘‚‘{Style.RESET_ALL}',
        f'  {Fore.BLUE}‚‘‚{Style.RESET_ALL}//{Fore.BLUE}‘‚‘‚{Style.RESET_ALL}/{Fore.BLUE}‘‚‘ {Style.RESET_ALL}',
        f' {Fore.BLUE}‚‘‚{Style.RESET_ALL}/{Fore.BLUE}‘‚‘‚‘{Style.RESET_ALL}/{Fore.BLUE}‚‘   {Style.RESET_ALL}'
    ],
    'new-moon-north': [
        '*    .                ',
        '        *             ',
        "             '        ",
        '            .        *',
        '                  .   '
    ],
    'new-moon-south': [
        '           *      ',
        '                 .',
        "          *     ' ",
        '                  ',
        '   @            * '
    ],
    'day-partial': [
        f'{BRIGHTYELLOW}   \\  /     {Style.RESET_ALL}',
        f'{BRIGHTYELLOW}__ /¯¯{Style.RESET_ALL}.-.   ',
        f'{BRIGHTYELLOW}   \\_{Style.RESET_ALL}(   ). ',
        f'{BRIGHTYELLOW}   /{Style.RESET_ALL}(___(__)',
    ],
    'day-clear': [
        f'{BRIGHTYELLOW}   \\   /   {Style.RESET_ALL}',
        f'{BRIGHTYELLOW}    .-.    {Style.RESET_ALL}',
        f'{BRIGHTYELLOW}-- (   ) --{Style.RESET_ALL}',
        f'{BRIGHTYELLOW}    `-’    {Style.RESET_ALL}',
        f'{BRIGHTYELLOW}   /   \\   {Style.RESET_ALL}'
    ],
    'day-clouds': [
        '    .--.   ',
        ' .-(    ). ',
        '(___.__)__)',
    ],
    'night-clouds': [
        '           ',
        f'{Fore.LIGHTBLACK_EX}    .--.   {Style.RESET_ALL}',
        f'{Fore.LIGHTBLACK_EX} .-(    ). {Style.RESET_ALL}',
        f'{Fore.LIGHTBLACK_EX}(___.__)__){Style.RESET_ALL}',
        '           '
    ],
    'rain-partial': [
        f'{BRIGHTYELLOW}_`/¯¯{Style.RESET_ALL}.-.   ',
        f'{BRIGHTYELLOW} ,\\_{Style.RESET_ALL}(   ). ',
        f'{BRIGHTYELLOW}  /{Style.RESET_ALL}(___(__)',
        f'{Fore.BLUE}  ‚‘‚‘‚‘‚‘ {Style.RESET_ALL}',
        f'{Fore.BLUE}  ‚’‚’‚’‚  {Style.RESET_ALL}'
    ],
    'day-snow': [
        '   .-.   ',
        '  (   ). ',
        ' (___(__)',
        ' * * * * ',
        '* * * *  '
    ],
    'night-snow': [
        f'{Fore.LIGHTBLACK_EX}   .-.   {Style.RESET_ALL}',
        f'{Fore.LIGHTBLACK_EX}  (   ). {Style.RESET_ALL}',
        f'{Fore.LIGHTBLACK_EX} (___(__){Style.RESET_ALL}',
        ' * * * * ',
        '* * * *  '
    ],
    'snow-partial': [
        f'{BRIGHTYELLOW}_`/¯¯{Style.RESET_ALL}.-.   ',
        f'{BRIGHTYELLOW} ,\\_{Style.RESET_ALL}(   ). ',
        f'{BRIGHTYELLOW}  /{Style.RESET_ALL}(___(__)',
        f'   *  *  * ',
        f'  *  *  *  '
    ],
    'night-thunderstorm': [
        f'      {Fore.LIGHTBLACK_EX}.--.   {Style.RESET_ALL}',
        f'   {Fore.LIGHTBLACK_EX}.-(    ). {Style.RESET_ALL}',
        f'  {Fore.LIGHTBLACK_EX}(___.__)__){Style.RESET_ALL}',
        f' {Fore.BLUE}‚‘{Style.RESET_ALL}/{Fore.BLUE}‚‘‚{Style.RESET_ALL}/_{Fore.BLUE}‘‚‘ {Style.RESET_ALL}',
        f'{Fore.BLUE}‚‘‚{Style.RESET_ALL}/{Fore.BLUE}‘‚‘‚{Style.RESET_ALL}/{Fore.BLUE}‘   {Style.RESET_ALL}'
    ],
    'day-thunderstorm': [
        f'      .--.   ',
        f'   .-(    ). ',
        f'  (___.__)__)',
        f' {Fore.BLUE}‚‘{Style.RESET_ALL}/{Fore.BLUE}‚‘‚{Style.RESET_ALL}/_{Fore.BLUE}‘‚‘ {Style.RESET_ALL}',
        f'{Fore.BLUE}‚‘‚{Style.RESET_ALL}/{Fore.BLUE}‘‚‘‚{Style.RESET_ALL}/{Fore.BLUE}‘   {Style.RESET_ALL}'
    ],
    'night-rain': [
        f'{Fore.LIGHTBLACK_EX}   .-.   {Style.RESET_ALL}',
        f'{Fore.LIGHTBLACK_EX}  (   ). {Style.RESET_ALL}',
        f'{Fore.LIGHTBLACK_EX} (___(__){Style.RESET_ALL}',
        f'{Fore.BLUE}‚‘‚‘‚‘‚‘ {Style.RESET_ALL}',
        f'{Fore.BLUE}‚’‚’‚’‚  {Style.RESET_ALL}',
    ],
    'day-rain': [
        f'   .-.   ',
        f'  (   ). ',
        f' (___(__)',
        f'{Fore.BLUE}‚‘‚‘‚‘‚‘ {Style.RESET_ALL}',
        f'{Fore.BLUE}‚’‚’‚’‚  {Style.RESET_ALL}',
    ]
}



import sys
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime
from time import time
from colorama import init, deinit
from wea.backend.openweather import get_urls, get_response_content
from wea.frontend.translations import location_prologue
from wea.backend.config import get_user_config
import wea.frontend.ui as ui

MORNING = [7, 8, 9]
NOON = [11, 12, 13]
EVENING = [17, 18, 19]
NIGHT = [21, 22, 23]
PARTS_OF_DAY = MORNING + NOON + EVENING + NIGHT


class Weather:
    """
    Class containing raw and formatted OpenWeather's API response.

    Basic usage::

        >>> weather = Weather(location, 'current')
        >>> weather.get_weather_response()
        >>> weather.handle_response()
        >>> weather.print_weather()
        Weather forecast for: Buenos Aires
           .-.     Light rain
          (   ).   24(24) °C
         (___(__)  →  3 km/h
        ‚‘‚‘‚‘‚‘   1006 hPa
        ‚’‚’‚’‚    81
                   10.0 km

    Attributes

    •response_list : list
        a list of JSON responses from OpenWeather, converted to dicts.
    •current : dict
        a formatted weather element.
    •forecast : list
        a list of formatted weather elements with weather conditions for the next 5 days.
    •city : dict
        OpenWeather's info about the location requested, containing sunrise and sunset hour.
    •weather_type : str
        must be either 'current' or 'forecast'. Otherwise, both reports are generated.
    """

    response_list = []
    current = {}
    forecast = []
    weather_type = ''
    city = {}

    def __init__(self, location):
        """
        Generates an empty Weather object.

        The location parameter must be a dict of the form
        { 'coord': [-34.6075682, -58.4370894], 'name': 'Buenos Aires', 'timezone_offset': -10800.0 }

        :param location: a dict containing the location's name, coordinates and timezone offset.
        :type location: dict
        """
        self.location = location
        config = get_user_config()
        self.lang = config['Preferences']['lang']
        self.units = config['Preferences']['units']
        self.api_key = config['API']['api_key']

    def get_weather_response(self):
        """
        Initializes self.response_list with the OpenWeather API\'s response.
        """
        config = get_user_config()
        api_key = config['API']['api_key']

        if 'coord' not in self.location:
            print('Please, set a location using wea.py -l {location} or provide one as an argument: wea {location}')
            sys.exit()
        elif not api_key:
            print('Please, provide your OpenWeather API key. You can get one for free with an OpenWeather account: '
                  'https://home.openweathermap.org/api_keys')
            sys.exit()
        else:
            timemachine_url1, timemachine_url2, forecast_url = get_urls(self.location)
            urls = [timemachine_url2, forecast_url]
            if timemachine_url1 != timemachine_url2:
                urls = [timemachine_url1] + urls

            if self.weather_type == 'current':
                self.response_list = [get_response_content(timemachine_url2)]
            else:
                with ThreadPoolExecutor(max_workers=3) as pool:
                    self.response_list = list(pool.map(get_response_content, urls))

    def handle_response(self):
        """ Initializes self.current and self.forecast if API\'s response exists. """

        if not self.response_list:
            print('Could not find API\'s response. Did you use Weather.get_weather_response()?')
            sys.exit()

        timemachine_list = self.response_list[0]['hourly'] + (
            self.response_list[1]['hourly'] if len(self.response_list) > 3 else [])
        if self.weather_type == 'current' or not self.weather_type:
            self.current = self.response_list[0]['current']
        if self.weather_type == 'forecast' or not self.weather_type:
            self.forecast = timemachine_list + self.response_list[-1]['list']
            self.city = self.response_list[-1]['city']

        if self.current:
            self.handle_current()

        if self.forecast:
            self.handle_forecast()

    def print_weather(self):
        """ Prints existing weather reports. """

        init()
        if not self.current and not self.forecast:
            print('Could not find any weather report to print. Did you use Weather.handle_response()?')
            sys.exit()

        print(location_prologue[self.lang], self.location['name'].title())
        if self.current:
            print(*ui.get_weather_card(self.current), sep='\n')
        if self.forecast:
            ui.print_forecast(self.forecast)
        deinit()

    def handle_current(self):
        timezone = self.location['timezone_offset']
        self.current['timezone_offset'] = self.location['timezone_offset']
        self.current['sunrise_hour'] = int(
            datetime.utcfromtimestamp(self.current['sunrise'] + timezone).strftime('%H'))
        self.current['sunset_hour'] = int(
            datetime.utcfromtimestamp(self.current['sunset'] + timezone).strftime('%H'))

        self.current = ui.format_weather_elem(self.current)

    def handle_forecast(self):
        timezone = self.location['timezone_offset']
        uncovered_pod = [NIGHT, EVENING, NOON, MORNING]
        first_forecast = next(elem.copy() for elem in self.forecast if 'main' in elem)
        first_forecast_hour = int(datetime.utcfromtimestamp(first_forecast['dt'] + timezone).strftime('%H'))

        new_list = []
        # Filters forecast elements
        for elem in self.forecast:
            elem['timezone_offset'] = timezone
            elem['lang'] = self.lang
            elem['sunrise_hour'] = int(datetime.utcfromtimestamp(self.city['sunrise'] + timezone).strftime('%H'))
            elem['sunset_hour'] = int(datetime.utcfromtimestamp(self.city['sunset'] + timezone).strftime('%H'))

            elem_date = datetime.utcfromtimestamp(elem['dt'] + timezone).date()
            elem_hour = int(datetime.utcfromtimestamp(elem['dt'] + timezone).strftime('%H'))
            local_date = datetime.utcfromtimestamp(time() + timezone).date()

            if elem_date == local_date:
                # Covering today's weather with data from timemachine's API
                if uncovered_pod and elem_hour in uncovered_pod[-1]:
                    new_list.append(ui.format_weather_elem(elem))
                    uncovered_pod.pop()

            elif elem_date > local_date:
                # Fills possible gap between timemachine and forecast responses
                if uncovered_pod and first_forecast_hour not in uncovered_pod[-1]:
                    new_list.append(self.current)
                    uncovered_pod.clear()

                # Default control logic for self.ther elem
                elif elem_hour in PARTS_OF_DAY:
                    new_list.append(ui.format_weather_elem(elem))

        self.forecast = new_list



import re
from datetime import datetime
from babel.dates import format_date
from colorama import Style, Fore
from wea.frontend.art import ASCII_ART, BRIGHTYELLOW
from wea.backend.config import get_user_config
from wea.frontend.translations import parts_of_day

WEATHER_UNITS = {
    'default': {
        'temp': 'K',
        'wind': 'km/h',
        'visibility': 'km'
    },
    'metric': {
        'temp': '°C',
        'wind': 'km/h',
        'visibility': 'km'
    },
    'imperial': {
        'temp': '°F',
        'wind': 'mph',
        'visibility': 'mi.'
    }
}


class Temperature:
    """ Formats temperature data. """

    def __init__(self, temp, feels_like, unit):
        self.temp = temp
        self.feels_like = feels_like
        self.unit = unit

    def __str__(self):
        return f'{self.get_intensitiy_color(self.temp)}{self.temp}{Style.RESET_ALL}' \
               f'({self.get_intensitiy_color(self.feels_like)}{self.feels_like}{Style.RESET_ALL}) {self.unit}'

    def get_intensitiy_color(self, temp):
        if self.unit == WEATHER_UNITS['default']['temp']:
            temp = temp - 273.15
        elif self.unit == WEATHER_UNITS['imperial']['temp']:
            temp = int((temp - 32) * 5 / 9)

        if temp < -15:
            return Fore.LIGHTBLUE_EX
        if temp < 0:
            return Fore.LIGHTMAGENTA_EX
        if temp < 6:
            return Fore.LIGHTWHITE_EX
        if temp < 16:
            return Style.RESET_ALL
        if temp < 26:
            return BRIGHTYELLOW
        if temp < 33:
            return Fore.LIGHTGREEN_EX
        else:
            return Fore.RED


class Wind:
    """ Formats wind data. """

    def __init__(self, deg, speed, unit):
        self.deg = deg
        self.speed = int(speed * (3.6 if unit != WEATHER_UNITS['imperial']['wind'] else 1))
        self.unit = unit

    def __str__(self):
        return f'{self.get_wind_direction()}  ' \
               f'{self.get_intensitiy_color(self.speed)}{self.speed}{Style.RESET_ALL} {self.unit}'

    def get_wind_direction(self):
        deg = self.deg
        if 340 <= deg or deg < 20:
            return '↓'
        elif 20 <= deg < 70:
            return '↙'
        elif 70 <= deg < 110:
            return '←'
        elif 110 <= deg < 160:
            return '↖'
        elif 160 <= deg < 200:
            return '↑'
        elif 200 <= deg < 250:
            return '↗'
        elif 250 <= deg < 290:
            return '→'
        elif 290 <= deg < 340:
            return '↘'

    def get_intensitiy_color(self, speed):
        if self.unit == WEATHER_UNITS['imperial']['wind']:
            speed = speed * 1.609
        speed = int(speed)

        if 5 <= speed < 16:
            return BRIGHTYELLOW
        if 16 <= speed < 25:
            return Fore.LIGHTGREEN_EX
        if 25 <= speed < 30:
            return Fore.GREEN
        if 30 <= speed:
            return Fore.RED
        else:
            return ''


def format_weather_elem(weather_elem):
    """
    Formats weather conditions to make them printable.

    :param weather_elem: a dictionary with relevant weather conditions.
    :type weather_elem: dict
    """

    config = get_user_config()
    units = config['Preferences']['units']

    # Reorganizes fields
    if 'main' in weather_elem:
        for prop in weather_elem['main']:
            weather_elem[prop] = weather_elem['main'][prop]
        weather_elem['wind_speed'] = weather_elem['wind']['speed']
        weather_elem['wind_deg'] = weather_elem['wind']['deg']

    # Sets temperature
    temp = Temperature(int(weather_elem['temp']), int(weather_elem['feels_like']), WEATHER_UNITS[units]['temp'])
    weather_elem['temp'] = temp.__str__()
    temp_width = len(weather_elem['temp']) - get_escapes_length(weather_elem['temp'])

    # Sets pressure
    weather_elem['pressure'] = f'{weather_elem["pressure"]} hPa'
    pressure_width = len(weather_elem['pressure'])

    # Sets rain data
    rainfall_rate = 0
    if 'rain' in weather_elem:
        if '1h' in weather_elem['rain']:
            rainfall_rate = weather_elem['rain']['1h']
        elif '3h' in weather_elem['rain']:
            rainfall_rate = weather_elem['rain']['3h']
    weather_elem['rain'] = f'{rainfall_rate} mm'
    if 'pop' in weather_elem:
        weather_elem['rain'] += f' | {int(weather_elem["pop"] * 100)}%'
    rain_width = len(weather_elem['rain'])

    # Sets wind data
    wind = Wind(weather_elem['wind_deg'], weather_elem['wind_speed'], WEATHER_UNITS[units]['wind'])
    weather_elem['wind'] = wind.__str__()
    wind_width = len(weather_elem['wind']) - get_escapes_length(weather_elem['wind'])

    # Sets visibility data
    conversion_factor_km2mi = 1.609
    visibility_km = weather_elem['visibility'] / 1000
    visibility_km = visibility_km if units != 'imperial' else visibility_km / conversion_factor_km2mi
    visibility_km = round(visibility_km, 1)
    weather_elem['visibility'] = str(visibility_km) + ' ' + WEATHER_UNITS[units]['visibility']

    description_width = len(weather_elem['weather'][0]['description'])
    weather_elem['max_width'] = max(description_width, temp_width, pressure_width, rain_width, wind_width)

    return weather_elem


def print_forecast(forecast):
    """
    Prints weather forecast to terminal.

    :param forecast: a list of formatted weather elements with weather conditions for the next 5 days.
    :type forecast: list
    """

    timezone = forecast[0]['timezone_offset']
    lang = forecast[0]['lang']

    forecast_day = []
    for elem in forecast:
        date = format_date(datetime.utcfromtimestamp(elem['dt'] + timezone), 'EEE d MMM', locale=lang)
        if len(date) > 13:
            date = format_date(date, format='short', locale=lang)

        weather_card = get_weather_card(elem)
        format_card(weather_card, len(forecast_day) == 3)
        forecast_day.append(weather_card)

        if len(forecast_day) == 4:
            forecast_day_display = get_weather_day(forecast_day, date)
            print(forecast_day_display)
            forecast_day.clear()


def get_weather_card(weather_state):
    """
    Generates a printable version of a weather element.

    For convenience, the weather card is not a string but a list of strings, each one representing a line of the
    actual card.
    """
    config = get_user_config()
    conditions_display = config['Conditions display']
    art = get_art(weather_state)

    max_width = weather_state['max_width']
    description = weather_state['weather'][0]['description']
    description = description.capitalize() + ' ' * (max_width - len(description))
    weather_conditions = [description]

    for prop in conditions_display:
        if conditions_display[prop]:
            weather_state[prop] = str(weather_state[prop])
            weather_state[prop] += ' ' * (max_width - len(weather_state[prop]) +
                                          get_escapes_length(weather_state[prop]))
            weather_conditions.append(weather_state[prop])

    return get_weather_card_content(weather_conditions, art)


def get_weather_card_content(weather_conditions, art_name):
    """
    Adds ascii art to a weather element card.

    :param weather_conditions: a list containing a description and formatted weather conditions.
    :type weather_conditions: list[str]
    :param art_name: name of the ascii icon to be used from art.py.
    :type art_name: str
    """
    art = ASCII_ART[art_name]

    art_length = len(art[0]) - get_escapes_length(art[0])
    art_rows = len(art)
    weather_rows = len(weather_conditions)
    weather_length = len(weather_conditions[0])

    rows = max(art_rows, weather_rows)
    blank_rows_top_art = int((rows - art_rows) / 2)

    if art_rows < weather_rows:
        art = [' ' * art_length] * blank_rows_top_art + \
              art + \
              [' ' * art_length] * (weather_rows - blank_rows_top_art - art_rows)
    elif art_rows > weather_rows:
        weather_conditions += [' ' * weather_length] * (art_rows - blank_rows_top_art - weather_rows)

    return list(map('  '.join, zip(art, weather_conditions)))


def get_weather_day(forecast_day, date):
    """
    Generates printable weather report for a whole day.

    :param forecast_day: a list of weather cards.
    :type forecast_day: list[list[str]]
    :param date: formatted date with a maximum length of 13 characters.
    :type date: str
    """
    config = get_user_config()
    lang = config['Preferences']['lang']
    pod = [center_word(parts_of_day[lang][i], 14) for i in range(0, 4)]

    date = center_word(date, 13)
    res = f'                                                       ┌─────────────┐                                                       \n' \
          f'┌──────────────────────────────┬───────────────────────│{    date   }│───────────────────────┬──────────────────────────────┐\n' \
          f'│        {   pod[0]   }        │        {   pod[1]   } └──────┬──────┘ {   pod[2]   }        │        {   pod[3]   }        │\n' \
          f'├──────────────────────────────┼──────────────────────────────┼──────────────────────────────┼──────────────────────────────┤\n' \

    forecast_content = list(map(''.join, zip(*forecast_day)))
    res += ''.join(forecast_content)
    res += (f'└──────────────────────────────┴──────────────────────────────┴──────────────────────────────┴──────────────────────────────┘')
    return res


def format_card(card, is_last):
    """
    Gives final format to a weather card.

    :param card: a weather card.
    :type card: list[str]
    :param is_last: if True, '│\n' will be added to the end of each line.
    :type is_last: bool
    """

    for i, line in enumerate(card):
        line = '│' + center_word(line, 30)
        escapes_length = get_escapes_length(line)
        line_width = len(line) - escapes_length
        if line_width > 31:
            line = line[0:30 + escapes_length] + ('…' if i == 0 else ' ')
        if is_last:
            line += '│\n'
        card[i] = line


def center_word(word, line_length):
    """
    Adds lateral spaces to center a word to fit in line of 
    :param word: 
    :param line_length: 
    :return: 
    """
    extra_spaces = line_length - len(word) + get_escapes_length(word)
    return ' ' * int((extra_spaces + 1) / 2) + word + ' ' * int(extra_spaces / 2)


def get_escapes_length(text):
    """ Returns that ANSI escapes code take in a string """
    escape_codes = re.findall(r'\x1b\[\d+;?\d?m', text)
    return sum(map(len, escape_codes))


def get_art(weather_state):
    codes = {
        2: 'thunderstorm',
        3: 'drizzle',
        5: 'rain',
        6: 'snow',
        7: 'mist',
        800: 'clear',
        801: 'partial',
        802: 'partial',
        803: 'partial',
        804: 'clouds'
    }
    code = int(weather_state['weather'][0]['id'])
    if str(code)[0] != '8':
        code = int(str(code)[0])
    condition = codes[code]
    if str(code)[0] != '7':
        local_hour = int(
            datetime.utcfromtimestamp(weather_state['dt'] + weather_state['timezone_offset']).strftime('%H'))
        if weather_state['sunset_hour'] < local_hour or local_hour < weather_state['sunrise_hour']:
            condition = 'night-' + condition
        else:
            condition = 'day-' + condition
    return condition



import re
import math
import itertools
from requests import get
from pytz import timezone, utc, UnknownTimeZoneError
from datetime import datetime
from timezonefinder.command_line import tf
from wea.backend import config
from wea.frontend import translations

BASE_URL = 'https://nominatim.openstreetmap.org/'


def get_geocoding_response(location=str):
    """
    Returns a list of geocoded locations.
    """
    location = re.findall(r'\w+', location)
    location = '+'.join(location)
    lang = config.get_user_config.__wrapped__()['Preferences']['lang']
    geocoded_locations = get(f'{BASE_URL}search.php?q={location}&format=jsonv2&accept-language={lang}').json()
    if not geocoded_locations:
        print(translations.unknown_location[lang])
        exit()
    return geocoded_locations


def get_places(location):
    """
    Generates a dictionary of locations geocoded by Nominatim.
    """
    places = {}
    cities = 10
    place_response = get_geocoding_response(location)
    delete_close_together(place_response, 10)
    for place in place_response:
        # Ensures that the places are not too broad
        # More about place ranking in https://nominatim.org/release-docs/develop/customize/Ranking/
        if place['place_rank'] >= cities:
            place_name = place['display_name']
            places[place_name] = format_location(place)
    return places


def delete_close_together(places, maximum_separation):
    """
    Filters places that are too close to each other, prioritazing the one with the smaller rank.

    :param places: a list of places generated by the geocoding tool from Nominatim.
    :param maximum_separation: maximum distance (in kilometers) for two places to be considered "close".
    """

    for place1, place2 in itertools.combinations(places[:], 2):
        coord1 = [float(place1['lat']), float(place1['lon'])]
        coord2 = [float(place2['lat']), float(place2['lon'])]
        if get_distance(coord1, coord2) < maximum_separation:
            rank_i = place1['place_rank']
            rank_j = place2['place_rank']
            place_greater_rank = place1 if rank_i > rank_j else place2
            if place_greater_rank in places:
                places.remove(place_greater_rank)


def format_location(location):
    location['coord'] = [float(location['lat']), float(location['lon'])]
    timezone_offset = get_offset(lat=location['coord'][0],
                                 lng=location['coord'][1])
    return {
        'name': location['display_name'].split(',')[0],
        'full_name': location['display_name'],
        'coord': location['coord'],
        'timezone_offset': timezone_offset
    }


def get_distance(coord1, coord2):
    """
    Returns the approximate distance between two geographical coordinates.
    """
    earth_radius_km = 6371
    lat1 = coord1[0] * math.pi / 180
    lon1 = coord1[1] * math.pi / 180
    lat2 = coord2[0] * math.pi / 180
    lon2 = coord2[1] * math.pi / 180

    # Haversine formula
    a = math.sin(abs(lat1 - lat2) / 2) ** 2 + math.cos(lat1) * math.cos(lat2) * \
        math.sin(abs(lon1 - lon2) / 2) ** 2
    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))

    return earth_radius_km * c


def get_offset(lat, lng):
    try:
        today = datetime.now()
        tz_target = timezone(tf.certain_timezone_at(lng=lng, lat=lat))
        today_target = tz_target.localize(today)
        today_utc = utc.localize(today)
        return (today_utc - today_target).total_seconds()

    except UnknownTimeZoneError:
        print(f'Couldn\'t determine the timezone for {lat} {lng}.')



import sys

from requests import get
from datetime import datetime
from time import time
from wea.backend.config import get_user_config

BASE_URL_FORECAST = 'https://api.openweathermap.org/data/2.5/forecast?'
BASE_URL_TIMEMACHINE = 'https://api.openweathermap.org/data/2.5/onecall/timemachine?'


def get_urls(location):
    """
    Returns a list of OpenWeather urls for the specified configuration.

    The location parameter must be a dict of the form
    { 'coord': [-34.6075682, -58.4370894], 'timezone_offset': -10800.0 }

    :param location: must contain the location's coordinates and timezone offset.
    :type location: dict
    """

    config = get_user_config()
    lat = location['coord'][0]
    lon = location['coord'][1]
    units = config['Preferences']['units']
    lang = config['Preferences']['lang'][0:2]
    api_key = config['API']['api_key']
    timezone = location['timezone_offset']

    current_time = int(time() - 1)
    utc_day = datetime.utcfromtimestamp(int(current_time)).strftime('%d')
    local_date = datetime.utcfromtimestamp(int(current_time) + timezone).strftime('%d')

    if local_date != utc_day and timezone < 0:
        request_time = int(current_time + timezone)
    else:
        request_time = current_time

    return [f'{BASE_URL_TIMEMACHINE}lat={lat}&lon={lon}&units={units}&lang={lang}&dt={request_time}&appid={api_key}',
            f'{BASE_URL_TIMEMACHINE}lat={lat}&lon={lon}&units={units}&lang={lang}&dt={current_time}&appid={api_key}',
            f'{BASE_URL_FORECAST}lat={lat}&lon={lon}&units={units}&lang={lang}&appid={api_key}']


def get_response_content(url):
    """
    Converts the response of the GET method to a dictionary.

    :param url: must provide json-encoded content when requested.
    :rtype: dict
    """

    response = get(url)
    if not response.ok:
        print(response.json()['message'])
        sys.exit()
    return response.json()



#!/usr/bin/env python3
import argparse
import sys
from wea.backend.weather import Weather
from wea.backend.config import set_location, set_unit_system, set_language, get_set_location, show_config, set_api_key
from wea.backend.nominatim import get_geocoding_response, format_location


parser = argparse.ArgumentParser()
group = parser.add_mutually_exclusive_group()
group.add_argument('-c', '--current', action='store_true',
                   help='Shows current weather for the set location.')
group.add_argument('-f', '--forecast', action='store_true',
                   help='Shows weather forecast for the set location.')
group.add_argument('-l', '--location', metavar='', type=str, action='store', nargs='+',
                   help='Sets a location.')
group.add_argument('-u', '--units', metavar='[default | metric | imperial]', action='store',
                   help='Changes unit system')
group.add_argument('--lang', metavar='', action='store', help='Sets new language.')
group.add_argument('-k', '--key', metavar='', action='store', help='Sets the OpenWeather API key.')
group.add_argument('--config', action='store_true', help='Shows current user configuration path and content.')


def main(args=None):

    try:

        if args is not None:
            sys.argv = [sys.argv[0] if len(sys.argv) > 0 else ''] + args.split()

        args, unknown = parser.parse_known_args(sys.argv)

        # User configuration
        if args.location:
            set_location(' '.join(args.location))
        elif args.units:
            set_unit_system(args.units)
        elif args.lang:
            set_language(args.lang)
        elif args.config:
            show_config()
        elif args.key:
            set_api_key(args.key)
        else:
            # Gets location if provided as an argument
            location = list(filter(lambda elem: elem[0] != '-', sys.argv[1:]))
            location = ' '.join(location)

            if not location:
                location = get_set_location()
            else:
                location = get_geocoding_response(location)[0]
                location = format_location(location)

            # Gets weather data and displays it
            weather = Weather(location)
            if args.current:
                weather.weather_type = 'current'
            elif args.forecast:
                weather.weather_type = 'forecast'
            weather.get_weather_response()
            weather.handle_response()
            weather.print_weather()

    except KeyboardInterrupt:
        print('Aborted by user.')
        sys.exit()


if __name__ == '__main__':
    main()



import json
import os
import sys
from difflib import get_close_matches
from pick import pick
from functools import lru_cache
from pprint import pprint
from wea.frontend import translations
from wea.backend import nominatim

ROOT = os.path.abspath(os.path.dirname(__file__))


@lru_cache(maxsize=1)
def get_user_config():
    """
    Returns user's configuration or creates its file if not found.

    :rtype: dict
    :except FileNotFoundError: if configuration file doesn't exist.
    """

    try:
        with open(os.path.join(ROOT, 'user_config.json'), 'r') as config_file:
            return json.loads(config_file.read())
    except FileNotFoundError:
        create_config_file()
        print('Configuration file created. Don\'t forget to set a location!')
        exit()


def get_set_location():
    return get_user_config()['Location']


def show_config():
    """
    Shows configuration file's path and content.
    """

    config = get_user_config()
    print('Path:', os.path.join(ROOT, 'user_config.json'))
    print('Configuration:')
    pprint(config)


def create_config_file():
    """
    Creates a default configuration file.
    """

    default_display = {'temp': True,
                       'wind': True,
                       'pressure': True,
                       'rain': True,
                       'visibility': True}
    config = {'API': {'api_key': ''},
              'Location': {},
              'Conditions display': default_display,
              'Preferences': {'lang': 'en', 'units': 'metric'}}
    with open(os.path.join(ROOT, 'user_config.json'), 'w') as config_file:
        json.dump(config, config_file, indent=4)


def add_config(new_config):
    """
    Updates user configuration.

    :param new_config: must be of the form { 'section': {'config element': 'new config'} }
    :type new_config: dict
    """

    config = get_user_config()

    for section in new_config:
        for key in new_config[section]:
            config[section][key] = new_config[section][key]

    with open(os.path.join(ROOT, 'user_config.json'), 'w') as config_file:
        json.dump(config, config_file, indent=4)


def set_location(location_in):
    """
    Lets user choose a location to update the configuration file.

    :param location_in: a textual description of the location.
    :type location_in: str
    """

    # Gets places from Nominatim
    possible_places = nominatim.get_places(location_in)
    places_names = list(possible_places.keys())

    config = get_user_config()
    lang = config['Preferences']['lang'][0:2]

    if not possible_places:
        print(f'{translations.unknown_location[lang]}')
        sys.exit()
    else:
        # Shows options
        option, chosen = pick(places_names, 'Choose a location: ')

    # Updates location
    location_name = places_names[chosen]

    add_config({'Location': possible_places[location_name]})
    print(f'{translations.location[lang]}: {places_names[chosen]}')


def set_unit_system(system):
    systems = ['default', 'metric', 'imperial']
    if system not in systems:
        print('Invalid unit system (available: default, metric, imperial).')
    else:
        new_config = {'Preferences': {'units': system}}
        add_config(new_config)
        print(f'New unit system: {system}.')


def set_language(lang):
    lang = lang.lower()
    config = get_user_config()
    current_lang = config['Preferences']['lang']
    location_name = config['Location']['full_name']
    if lang not in translations.lang_codes.keys():
        close_matches = get_close_matches(lang, translations.lang_codes.keys(), n=1)
        print('Invalid language' + (f', did you mean "{close_matches[0]}"?' if close_matches else '.'))
    elif current_lang not in [lang, translations.lang_codes[lang]]:
        new_config = {'Preferences': {'lang': translations.lang_codes[lang]}}
        add_config(new_config)
        location_new_language = nominatim.get_geocoding_response(location_name)[0]
        location_new_language = nominatim.format_location(location_new_language)
        add_config({'Location': location_new_language})

        print(f'New language: {lang}.')
    else:
        print('This language is already set.')


def set_api_key(api_key):
    new_config = {'API': {'api_key': api_key}}
    add_config(new_config)
    print('API key set.')



location_prologue = {
    'af': 'Weer verslag vir:',
    'ar': 'تقرير حالة ألطقس',
    'az': 'Hava proqnozu:',
    'bg': 'Прогноза за времето в:',
    'ca': 'Informe del temps per a:',
    'da': 'Vejret i:',
    'de': 'Wetterbericht für:',
    'el': 'Πρόγνωση καιρού για:',
    'es': 'El tiempo en:',
    'eu': 'Eguraldia:',
    'fa': 'اوه و بآ تیعضو شرازگ',
    'fi': 'Säätiedotus:',
    'fr': 'Prévisions météo pour:',
    'he': ':ריוואה גזמ תיזחת',
    'hi': 'मौसम की जानकारी',
    'hr': 'Vremenska prognoza za:',
    'hu': 'Időjárás előrejelzés:',
    'id': 'Prakiraan cuaca:',
    'it': 'Previsioni meteo:',
    'ja': '天気予報：',
    'en': 'Weather forecast for:',
    'lt': 'Orų prognozė:',
    'mk': 'Прогноза за времето во:',
    'nl': 'Weerbericht voor:',
    'pl': 'Pogoda w:',
    'pt': 'Previsão do tempo para:',
    'ro': 'Prognoza meteo pentru:',
    'ru': 'Прогноз погоды:',
    'sk': 'Predpoveď počasia pre:',
    'sl': 'Vremenska napoved za',
    'sr': 'Временска прогноза за:',
    'sv': 'Väderleksprognos för:',
    'th': 'รายงานสภาพอากาศ:',
    'tr': 'Hava beklentisi:',
    'uk': 'Прогноз погоди для:',
    'vi': 'Báo cáo thời tiết:',
    'zu': 'Isimo sezulu:'
}

parts_of_day = {
    'af': ('Oggend', 'Middag', 'Vroegaand', 'Laatnag'),
    'ar': ('ﺎﻠﻠﻴﻟ', 'ﺎﻠﻤﺳﺍﺀ', 'ﺎﻠﻈﻫﺭ', 'ﺎﻠﺼﺑﺎﺣ'),
    'az': ('Səhər', 'Gün', 'Axşam', 'Gecə'),
    'bg': ('Сутрин', 'Обяд', 'Вечер', 'Нощ'),
    'ca': ('Matí', 'Dia', 'Tarda', 'Nit'),
    'da': ('Morgen', 'Middag', 'Aften', 'Nat'),
    'de': ('Früh', 'Mittag', 'Abend', 'Nacht'),
    'el': ('Πρωί', 'Μεσημέρι', 'Απόγευμα', 'Βράδυ'),
    'en': ('Morning', 'Noon', 'Evening', 'Night'),
    'es': ('Mañana', 'Mediodía', 'Tarde', 'Noche'),
    'eu': ('Goiza', 'Eguerdia', 'Arratsaldea', 'Gaua'),
    'fa': ('حبص', 'رهظ', 'رصع', 'بش'),
    'fi': ('Aamu', 'Keskipäivä', 'Ilta', 'Yö'),
    'fr': ('Matin', 'Après-midi', 'Soir', 'Nuit'),
    'he': ('רקוב', 'םוֹיְ', 'ברֶעֶ', 'הלָיְלַ'),
    'hi': ('प्रातःकाल', 'दोपहर', 'सायंकाल', 'रात'),
    'hr': ('Jutro', 'Dan', 'Večer', 'Noć'),
    'hu': ('Reggel', 'Dél', 'Este', 'Éjszaka'),
    'id': ('Pagi', 'Hari', 'Petang', 'Malam'),
    'it': ('Mattina', 'Pomeriggio', 'Sera', 'Notte'),
    'ja': ('朝', '昼', '夕', '夜'),
    'lt': ('Rytas', 'Diena', 'Vakaras', 'Naktis'),
    'mk': ('Утро', 'Пладне', 'Вечер', 'Ноќ'),
    'nl': ("'s Ochtends", "'s Middags", "'s Avonds", "'s Nachts"),
    'pl': ('Ranek', 'Dzień', 'Wieczór', 'Noc'),
    'pt': ('Manhã', 'Meio-dia', 'Tarde', 'Noite'),
    'ro': ('Dimineaţă', 'Amiază', 'Seară', 'Noapte'),
    'ru': ('Утро', 'День', 'Вечер', 'Ночь'),
    'sk': ('Ráno', 'Cez deň', 'Večer', 'V noci'),
    'sl': ('Jutro', 'Dan', 'Večer', 'Noč'),
    'sr': ('Јутро', 'Подне', 'Вече', 'Ноћ'),
    'sv': ('Morgon', 'Eftermiddag', 'Kväll', 'Natt'),
    'th': ('เช้า', 'วัน', 'เย็น', 'คืน'),
    'tr': ('Sabah', 'Öğle', 'Akşam', 'Gece'),
    'uk': ('Ранок', 'День', 'Вечір', 'Ніч'),
    'vi': ('Sáng', 'Trưa', 'Chiều', 'Tối'),
    'zu': ('Morning', 'Noon', 'Evening', 'Night')
}

lang_names = {
    'af': {'name': 'Afrikaans', 'dir': 1, 'native': 'Afrikaans'},
    'ar': {'name': 'Arabic', 'dir': 0, 'native': 'العربية'},
    'az': {'name': 'Azerbaijani', 'dir': 1, 'native': 'Azərbaycanca / آذربايجان'},
    'bg': {'name': 'Bulgarian', 'dir': 1, 'native': 'Български'},
    'ca': {'name': 'Catalan', 'dir': 1, 'native': 'Català'},
    'da': {'name': 'Danish', 'dir': 1, 'native': 'Dansk'},
    'de': {'name': 'German', 'dir': 1, 'native': 'Deutsch'},
    'el': {'name': 'Greek', 'dir': 1, 'native': 'Ελληνικά'},
    'en': {'name': 'English', 'dir': 1, 'native': 'English'},
    'es': {'name': 'Spanish', 'dir': 1, 'native': 'Español'},
    'eu': {'name': 'Basque', 'dir': 1, 'native': 'Euskara'},
    'fa': {'name': 'Persian', 'dir': 0, 'native': 'فارسی'},
    'fi': {'name': 'Finnish', 'dir': 1, 'native': 'Suomi'},
    'fr': {'name': 'French', 'dir': 1, 'native': 'Français'},
    'he': {'name': 'Hebrew', 'dir': 0, 'native': 'עברית'},
    'hi': {'name': 'Hindi', 'dir': 1, 'native': 'हिन्दी'},
    'hr': {'name': 'Croatian', 'dir': 1, 'native': 'Hrvatski'},
    'hu': {'name': 'Hungarian', 'dir': 1, 'native': 'Magyar'},
    'id': {'name': 'Indonesian', 'dir': 1, 'native': 'Bahasa Indonesia'},
    'it': {'name': 'Italian', 'dir': 1, 'native': 'Italiano'},
    'ja': {'name': 'Japanese', 'dir': 1, 'native': '日本語'},
    'lt': {'name': 'Lithuanian', 'dir': 1, 'native': 'Lietuvių'},
    'mk': {'name': 'Macedonian', 'dir': 1, 'native': 'Македонски'},
    'nl': {'name': 'Dutch', 'dir': 1, 'native': 'Nederlands'},
    'pl': {'name': 'Polish', 'dir': 1, 'native': 'Polski'},
    'pt': {'name': 'Portuguese', 'dir': 1, 'native': 'Português'},
    'ro': {'name': 'Romanian', 'dir': 1, 'native': 'Română'},
    'ru': {'name': 'Russian', 'dir': 1, 'native': 'Русский'},
    'sk': {'name': 'Slovak', 'dir': 1, 'native': 'Slovenčina'},
    'sl': {'name': 'Slovenian', 'dir': 1, 'native': 'Slovenščina'},
    'sr': {'name': 'Serbian', 'dir': 1, 'native': 'Српски'},
    'sv': {'name': 'Swedish', 'dir': 1, 'native': 'Svenska'},
    'th': {'name': 'Thai', 'dir': 1, 'native': 'ไทย / Phasa Thai'},
    'tr': {'name': 'Turkish', 'dir': 1, 'native': 'Türkçe'},
    'uk': {'name': 'Ukrainian', 'dir': 1, 'native': 'Українська'},
    'vi': {'name': 'Vietnamese', 'dir': 1, 'native': 'Việtnam'},
    'zu': {'name': 'Zulu', 'dir': 1, 'native': 'isiZulu'}
}

lang_codes = {
    'afrikaans': 'af',
    'arabic': 'ar',
    'العربية': 'ar',
    'azerbaijani': 'az',
    'azərbaycanca': 'az',
    'آذربايجان': 'az',
    'bulgarian': 'bg',
    'български': 'bg',
    'catalan': 'ca',
    'català': 'ca',
    'danish': 'da',
    'dansk': 'da',
    'german': 'de',
    'deutsch': 'de',
    'greek': 'el',
    'ελληνικά': 'el',
    'english': 'en',
    'spanish': 'es',
    'español': 'es',
    'basque': 'eu',
    'euskara': 'eu',
    'persian': 'fa',
    'فارسی': 'fa',
    'finnish': 'fi',
    'suomi': 'fi',
    'french': 'fr',
    'français': 'fr',
    'hebrew': 'he',
    'עברית': 'he',
    'hindi': 'hi',
    'हिन्दी': 'hi',
    'croatian': 'hr',
    'hrvatski': 'hr',
    'hungarian': 'hu',
    'magyar': 'hu',
    'indonesian': 'id',
    'bahasa indonesia': 'id',
    'italian': 'it',
    'italiano': 'it',
    'japanese': 'ja',
    '日本語': 'ja',
    'lithuanian': 'lt',
    'lietuvių': 'lt',
    'macedonian': 'mk',
    'македонски': 'mk',
    'dutch': 'nl',
    'nederlands': 'nl',
    'polish': 'pl',
    'polski': 'pl',
    'portuguese': 'pt',
    'português': 'pt',
    'romanian': 'ro',
    'română': 'ro',
    'russian': 'ru',
    'русский': 'ru',
    'slovak': 'sk',
    'slovenčina': 'sk',
    'slovenian': 'sl',
    'slovenščina': 'sl',
    'serbian': 'sr',
    'српски': 'sr',
    'swedish': 'sv',
    'svenska': 'sv',
    'thai': 'th',
    'ไทย': 'th',
    'phasa thai': 'th',
    'turkish': 'tr',
    'türkçe': 'tr',
    'ukrainian': 'uk',
    'українська': 'uk',
    'vietnamese': 'vi',
    'việtnam': 'vi',
    'zulu': 'zu',
    'isizulu': 'zu'
}

location = {
    'en': 'Location',
    'af': 'Ligging',
    'ar': 'الموقع',
    'bg': 'Местоположение',
    'ca': 'Ubicació',
    'de': 'Ort',
    'da': 'Lokation',
    'el': 'Τοποθεσία',
    'es': 'Ubicación',
    'eu': 'Kokaena',
    'fa': 'مکان',
    'fi': 'Tuntematon sijainti',
    'fr': 'Emplacement',
    'hi': 'स्थान',
    'hu': 'Lokáció',
    'id': 'Lokasi',
    'it': 'Località',
    'ja': '位置情報',
    'lt': 'Vietovė',
    'mk': 'Локација',
    'nl': 'Locatie',
    'pl': 'Lokalizacja',
    'pt': 'Localização',
    'ro': 'Localitate',
    'ru': 'Местоположение',
    'sk': 'Poloha',
    'sl': 'Lokacijo',
    'sr': 'Локација',
    'sv': 'Plats',
    'tr': 'Konum',
    'th': 'ตำแหน่ง',
    'uk': 'Місцезнаходження',
    'vi': 'Địa điểm'
}

unknown_location = {
    'en': 'Unknown location',
    'af': 'Onbekende ligging',
    'ar': 'موقع غير معروف',
    'bg': 'Неизвестно местоположение',
    'ca': 'Ubicació desconeguda',
    'de': 'Unbekannter Ort',
    'da': 'Ukendt lokation',
    'el': 'Άνγωστη τοποθεσία',
    'es': 'Ubicación desconocida',
    'eu': 'Kokapen ezezaguna',
    'fa': 'مکان نامعلوم',
    'fi': 'Tuntematon sijainti',
    'fr': 'Emplacement inconnu',
    'hi': 'अज्ञात स्थान',
    'hu': 'Ismeretlen lokáció',
    'id': 'Lokasi tidak diketahui',
    'it': 'Località sconosciuta',
    'ja': '未知の場所です',
    'lt': 'Nežinoma vietovė',
    'mk': 'Непозната локација',
    'nl': 'Onbekende locatie',
    'pl': 'Nieznana lokalizacja',
    'pt': 'Localização desconhecida',
    'ro': 'Localitate necunoscută',
    'ru': 'Неизвестное местоположение',
    'sk': 'Neznáma poloha',
    'sl': 'Neznano lokacijo',
    'sr': 'Непозната локација',
    'sv': 'Okänd plats',
    'tr': 'Bilinmeyen konum',
    'th': 'ไม่สามารถระบุตำแหน่งได้',
    'uk': 'Невідоме місце',
    'vi': 'Địa điểm không xác định'
}
